CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  SI = 0                       ' Camera SI
  TSL_CLK = 1                  ' Camera clock
  ADC_CLK     = 3              ' ADC Clock
  CS_BAR_PIN  = 2              ' ADC chip select
  DO_PIN      = 4              ' ADC data out pin
  CNT_MIN     = 400            ' Minimum waitcnt value to prevent lock-up
OBJ
     Sio  : "FullDuplexSerialPlus"        

VAR                                     '
  long exp                      ' exposure time in micro seconds
  long image[128]               ' image values
  long uS                       ' clock cycles for a micro second
Pub Main   | i
  Initialize
  Sio.start(31,30,0,115200)  ' Rx,Tx, Mode, Baud
'  Sio.start(8,9,0,115200)  ' Rx,Tx, Mode, Baud      XBee
  exp := 100000                   ' 1/10 second
  repeat
    GetImage
    Sio.tx(0)
    repeat i from 0 to 127
      Sio.dec(image[i])
      Sio.tx(13)
    repeat until !Sio.rxcheck                            ' wait for key from terminal
        
PUB GetImage | i, data                    ' one shot image acquire
  outa[SI]~~
  outa[TSL_CLK]~~
  outa[SI]~
  outa[TSL_CLK]~
  repeat 250
    !outa[TSL_CLK]
  outa[TSL_CLK]~
  Pause_uS(exp)
  outa[SI]~~
  outa[TSL_CLK]~~
  outa[SI]~
  outa[TSL_CLK]~
  outa[TSL_CLK]~~               '**** Don't know why I need this pulse.  If I don't the                        
  outa[TSL_CLK]~                '**** first value is always 255
  repeat i from 0 to 127
    outa[TSL_CLK]~~                                                ' high
    image[i] := 0                     ' This will accumulate the resulting value
    outa[CS_BAR_PIN]~             ' sets pin -CS low to activate the chip    
     
    outa[ADC_CLK]~~               ' pulse the clock, first high      
    outa[ADC_CLK]~                ' then low, this starts the conversion
     
    'Read 8 bits, MSB first.
    ' Althoug the datasheet I have says that LSB is not available on the ADC0831, it is present on my chip
    repeat 8
      image[i] <<= 1                  ' Multiply data by two
      outa[ADC_CLK]~~             ' pulse the clock, first high      
      outa[ADC_CLK]~              ' then low, this makes the next bit available on DO
      image[i] += ina[DO_PIN]         ' Add it to the current value            
    outa[CS_BAR_PIN]~~            ' Terminated, deselect the chip
    outa[TSL_CLK]~                                                 ' low

PUB Initialize
{{ Initialize the pins direction and state. Must be called once. }}
  dira[DO_PIN]~                 ' set DO pin as input (is default at start, present for clarity)

  outa[CS_BAR_PIN]~~            ' sets pin -CS high  (you must always first set the value, then enable the output!)
  dira[CS_BAR_PIN]~~            ' sets -CS pin as output (this means disabled !)
 
  outa[ADC_CLK]~                ' sets clock pin low (is default at start, present for clarity)
  dira[ADC_CLK]~~               ' sets pin as output

  dira[TSL_CLK]~~
  dira[SI]~~
  outa[TSL_CLK]~
  outa[SI]~
  us:= clkfreq / 1_000_000                  ' Clock cycles for 1 us 

PRI ReadADC | data         ' data could also be a byte VARiable
{{ Aquiring data requires to assert CS, pulse the clock once to start aquisition, then
   pulse the clock 8 times to read each bit after the descending edge of the clock.
  The chip is driven by the clock signal we generate, so timing is not critical. }}
  
  return data

PRI PAUSE_uS(Duration) | clkCycles
{{
   Causes a pause for the duration in uS
   Smallest value is 20 at clkfreq = 80Mhz
   Largest value is around 50 seconds at 80Mhz.
     BS2.Pause_uS(1000)   ' 1 mS pause
}}
   clkCycles := Duration * uS #> CNT_MIN                    ' duration * clk cycles for us
                                                           ' - inst. time, min cntMin 
   waitcnt(clkcycles + cnt)                                ' wait until clk gets there                             
